import { BlockPermutation, world } from "@minecraft/server";

const cardinalSides = Object.freeze({
    north: { left: 'east', right: 'west', },
    south: { left: 'west', right: 'east', },
    west: { left: 'north', right: 'south', },
    east: { left: 'south', right: 'north', }
});

world.beforeEvents.worldInitialize.subscribe((data) => {
    data.blockTypeRegistry.registerCustomComponent('door:toggle', {
        onPlayerInteract(event) {
            const {block} = event;
            const isOpen = block.permutation.getState('door:is_open');
            const getHalfDoor = () => {
                switch(block.permutation.getState('door:part')) {
                    case 'lower': return block.above();
                    case 'upper': return block.below();
                }
            }
            const halfDoor = getHalfDoor();

            block.setPermutation(block.permutation.withState('door:is_open', !isOpen));
            block.dimension.playSound(isOpen ? 'random.door_close' : 'random.door_open', block.location, {pitch: 1.0, volume: 0.4});
            if(halfDoor?.permutation.matches(block.typeId)) {
                halfDoor.setPermutation(halfDoor.permutation.withState('door:is_open', !isOpen));
            }
        }
    });

    data.blockTypeRegistry.registerCustomComponent('door:on_place', {
        beforeOnPlayerPlace(event) {
            const {block, permutationToPlace, player} = event;
            const aboveBlock = block.above();
            const canBePlace = aboveBlock.isAir || aboveBlock.isLiquid;

            if(canBePlace) {
                const hasSideDoor = (sideBlock) => {
                    return sideBlock.matches(permutationToPlace.type.id, {
                        ...sideBlock.permutation.getAllStates(),
                        'door:part': 'lower'
                    });
                }

                const cardinalDirection = permutationToPlace.getState('minecraft:cardinal_direction');
                const sides = cardinalSides[cardinalDirection];
                const leftBlock = block[sides.left]();
                const rightBlock = block[sides.right]();
                const isSideDoor = hasSideDoor(leftBlock) && !hasSideDoor(rightBlock);
                const hasRightBlock = rightBlock.isSolid || rightBlock.above().isSolid;
                const doorStates = {
                    'minecraft:cardinal_direction': cardinalDirection,
                    'door:side': isSideDoor || hasRightBlock ? 'right': 'left',
                }

                if(permutationToPlace.getState('door:top_texture') !== undefined) {
                    doorStates['door:top_texture'] = player.isSneaking;
                }

                const doorPermutation = BlockPermutation.resolve(permutationToPlace.type.id, doorStates);

                event.permutationToPlace = doorPermutation.withState('door:part', 'lower');
                aboveBlock.setPermutation(doorPermutation.withState('door:part', 'upper'));
            }
            else {
                event.cancel = true;
            }
        }
    });

    data.blockTypeRegistry.registerCustomComponent('door:changeable_texture', {
        beforeOnPlayerPlace(event) {
            const {player, permutationToPlace} = event;
            event.permutationToPlace = permutationToPlace.withState('door:top_texture', player.isSneaking);
        }
    });
});